using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MifareDemo
{
    public partial class MainForm : Form
    {        
        [DllImport("Coredll.dll", SetLastError = true)]
        private static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport("Coredll.dll")]
        private static extern int SetWindowLong(IntPtr hWnd, int nIndex, int nNew);

        private const int GWL_ID = (-12);
        private const int GWL_STYLE = (-16);
        private const int GWL_EXSTYLE = (-20);

        private const int ES_UPPERCASE = 0x0008;
        private const int ES_LOWERCASE = 0x0010;

        private Char[] hexChars = new Char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', '\x08'};

        public MainForm()
        {
            InitializeComponent();
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {
            textBoxLog.Text = "";
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            int nStyle = GetWindowLong(textBoxKeyValue.Handle, GWL_STYLE);
            SetWindowLong(textBoxKeyValue.Handle, GWL_STYLE, nStyle | ES_UPPERCASE);

            comboBoxKeyType.SelectedIndex = 0;
        }
        
        private void textBoxKeyValue_KeyPress(object sender, KeyPressEventArgs e)
        {                        
            if (Array.IndexOf(hexChars, Char.ToUpper(e.KeyChar)) == -1)
                e.Handled = true;
        }

        private void Log(string strText)
        {
            textBoxLog.SelectionStart = textBoxLog.Text.Length;
            textBoxLog.SelectionLength = 0;
            textBoxLog.SelectedText = strText + "\r\n";
            textBoxLog.SelectionStart = textBoxLog.Text.Length;
            textBoxLog.SelectionLength = 0;
            textBoxLog.ScrollToCaret();
            textBoxLog.Update();
        }

        private Byte[] HexToByteArray(String strHex)
        {
            int nChars = strHex.Length;
            Byte[] buffer = new Byte[nChars / 2];

            for (int i = 0; i < nChars; i += 2)
            {
                buffer[i / 2] = Convert.ToByte(strHex.Substring(i, 2), 16);
            }

            return buffer;
        }        

        private void buttonCkeck_Click(object sender, EventArgs e)
        {
            PrintDriverNet.Printer printer = new PrintDriverNet.Printer();
            PrintDriverNet.Printer.EMifareKeyType keyType;
            Byte[] key = HexToByteArray(textBoxKeyValue.Text);
            Byte[] buffer = new Byte[16];
            Byte cStatus;
            int iResult, i;

            if (comboBoxKeyType.SelectedIndex == 0)
                keyType = PrintDriverNet.Printer.EMifareKeyType.KEYA;
            else
                keyType = PrintDriverNet.Printer.EMifareKeyType.KEYB;

            try
            {
                if (key.Length != 6)
                    throw new Exception("Invalid key.");

                Log("Connect to printer...");

                iResult = printer.OpenPrinter();
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to open connection with printer ({0}).", iResult));

                Log("Init mifare reader...");
                iResult = printer.OpenChannel(PrintDriverNet.Printer.EChannels.Mifare);
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to open mifare channel (Result: {0}).", iResult));

                iResult = printer.Mifare_PowerOn();
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to power on mifare reader (Result: {0}).", iResult));

                iResult = printer.Mifare_Config(out cStatus);
                if ((iResult < 0) || (cStatus != 0))
                    throw new Exception(String.Format("Failed to config on mifare reader (Result: {0}, Status: {1}).", iResult, cStatus));

                Log("Search for mifare card...");

                while (true)
                {
                    Byte cRq1, cRq2, cSAck;
                    uint nSerialNumber;

                    Log("");

                    iResult = printer.Mifare_Request(PrintDriverNet.Printer.EMifareRequestType.IdleCards, out cStatus, out cRq1, out cRq2);
                    if (iResult < 0)
                        throw new Exception(String.Format("Failed to request for mifare card (Result: {0}).", iResult));
                    if (cStatus != 0)
                    {
                        Log("No more cards found.");
                        break; // Not card in range 
                    }

                    iResult = printer.Mifare_Anticollision(out cStatus, out nSerialNumber);
                    if ((iResult < 0) || (cStatus != 0)) 
                        throw new Exception(String.Format("Failed to find mifare card (Result: {0}, Status: {1}).", iResult, cStatus));

                    iResult = printer.Mifare_SelectCard(nSerialNumber, out cStatus, out cSAck);
                    if ((iResult < 0) || (cStatus != 0))
                        throw new Exception(String.Format("Failed to select mifare card (Result: {0}, Status: {1}).", iResult, cStatus));
                    
                    Log("CARD: " + nSerialNumber.ToString("X8"));
                    Log("--------------");
                                        
                   for (i = 0; i < numericUpDownBlocks.Value; i++)
                    {
                        if ((i % 4) == 0)
                        {
                            iResult = printer.Mifare_AuthByKey(nSerialNumber, keyType, i, key, out cStatus);
                            if (iResult < 0)
                                throw new Exception(String.Format("Failed to authenticate sector in mifare card (Result: {0}).", iResult));

                            if (cStatus == 0)
                                Log(String.Format("  Auth sector {0}... OK", (i / 4)));
                            else
                                Log(String.Format("  Auth sector {0}... FAILED", (i / 4)));                            
                        }

                        iResult = printer.Mifare_Read(i, out cStatus, buffer);
                        if (iResult < 0)
                            throw new Exception(String.Format("Failed to read block from mifare card (Result: {0}).", iResult));

                        if (cStatus == 0)
                        {
                            Log(String.Format("  Read block {0}... OK", i));
                            Log(String.Format("    --> {0:X2} {1:X2} {2:X2} {3:X2}", buffer[0], buffer[1], buffer[2], buffer[3]));
                            Log(String.Format("    --> {0:X2} {1:X2} {2:X2} {3:X2}", buffer[4], buffer[5], buffer[6], buffer[7]));
                            Log(String.Format("    --> {0:X2} {1:X2} {2:X2} {3:X2}", buffer[8], buffer[9], buffer[10], buffer[11]));
                            Log(String.Format("    --> {0:X2} {1:X2} {2:X2} {3:X2}", buffer[12], buffer[13], buffer[14], buffer[15]));
                        }
                        else
                            Log(String.Format("  Read block {0}... FAILED", i));
                    }

                    Log("--------------");

                    iResult = printer.Mifare_Halt(out cStatus);
                    if ((iResult < 0) || (cStatus != 0))
                        throw new Exception(String.Format("Failed to select mifare card (Result: {0}, Status: {1}).", iResult, cStatus));
                }

                iResult = printer.Mifare_PowerOff();
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to power off mifare reader ({0}).", iResult));

                iResult = printer.CloseChannel(PrintDriverNet.Printer.EChannels.Mifare);
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to close mifare channel ({0}).", iResult));

                iResult = printer.ClosePrinter();
                if (iResult < 0)
                    throw new Exception(String.Format("Failed to close connection with printer ({0}).", iResult));

                Log("Finish.");                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
            finally
            {
                printer.Dispose();
                Log("");
            }
        }
    }
}