Imports PrintDriverNet

Public Class MainForm

    Private normalText As String = "Sample text to print."
    Private taggedText As String = _
        "[FI]" & Chr(13) & Chr(10) & _
        "[FA][DW+][DH+]   LARGE FONT[DW-][DH-][LF]" & Chr(13) & Chr(10) & _
        "[LF]" & Chr(13) & Chr(10) & _
        "Normal[LF]" & Chr(13) & Chr(10) & _
        "[B+]Bold[B-][LF]" & Chr(13) & Chr(10) & _
        "[I+]Italic[I-][LF]" & Chr(13) & Chr(10) & _
        "[U+]Underlined[U-][LF]" & Chr(13) & Chr(10) & _
        "[DW+][DH+]Diff[DH-]erent[DW-] [DH+]sizes[DH-] chars[LF]" & Chr(13) & Chr(10) & _
        "[LF][LF]" & Chr(13) & Chr(10) & _
        "[FB][DW+][DH+]      SMALL FONT[DW-][DH-][LF]" & Chr(13) & Chr(10) & _
        "[LF]" & Chr(13) & Chr(10) & _
        "Normal[LF]" & Chr(13) & Chr(10) & _
        "[B+]Bold[B-][LF]" & Chr(13) & Chr(10) & _
        "[I+]Italic[I-][LF]" & Chr(13) & Chr(10) & _
        "[U+]Underlined[U-][LF]" & Chr(13) & Chr(10) & _
        "[DW+][DH+]Diff[DH-]erent[DW-] [DH+]sizes[DH-] chars[LF]" & Chr(13) & Chr(10) & _
        "[FA][LF][LF][LF]"

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Print text
        TextBox.Text = normalText

        ' Print barcode
        comboBoxType.Items.Clear()
        comboBoxType.Items.Add("UPCA") ' 0
        comboBoxType.Items.Add("UPCE") ' 1
        comboBoxType.Items.Add("EAN13") '  2
        comboBoxType.Items.Add("EAN8") ' 3
        comboBoxType.Items.Add("CODE39") '  4
        comboBoxType.Items.Add("ITF") '  5
        comboBoxType.Items.Add("CODABAR") '  6
        comboBoxType.Items.Add("CODE93") '  7
        comboBoxType.Items.Add("CODE128") '  8
        comboBoxType.Items.Add("PDF417") '  9
        comboBoxType.Items.Add("CODE128AUTO") '  10
        comboBoxType.Items.Add("EAN128AUTO") '  11
        comboBoxType.SelectedIndex = 0

        textBoxScale.Text = "2"
        textBoxHeight.Text = "90"

        comboBoxTextPosition.Items.Clear()
        comboBoxTextPosition.Items.Add("NoPrint")
        comboBoxTextPosition.Items.Add("Above")
        comboBoxTextPosition.Items.Add("Below")
        comboBoxTextPosition.Items.Add("AboveAndBelow")
        comboBoxTextPosition.SelectedIndex = 0

        comboBoxTextAlign.Items.Clear()
        comboBoxTextAlign.Items.Add("Left")
        comboBoxTextAlign.Items.Add("Center")
        comboBoxTextAlign.Items.Add("Right")
        comboBoxTextAlign.Items.Add("LeftVert")
        comboBoxTextAlign.Items.Add("CenterVert")
        comboBoxTextAlign.Items.Add("RightVert")
        comboBoxTextAlign.SelectedIndex = 0

        comboBoxTextSize.Items.Clear()
        comboBoxTextSize.Items.Add("FONTA")
        comboBoxTextSize.Items.Add("FONTB")
        comboBoxTextSize.SelectedIndex = 0
    End Sub

    Private Sub MenuItemExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemExit.Click
        Close()
    End Sub

    Private Sub CheckBoxTagged_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxTagged.Click
        If CheckBoxTagged.Checked Then
            TextBox.Text = taggedText
        Else
            TextBox.Text = normalText
        End If
    End Sub

    Private Sub ButtonPrintDirect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrintDirect.Click
        Dim printer As New Printer
        Dim buffer As Byte()

        Try
            If printer.OpenPrinter(False, "") < 0 Then
                Throw New Exception("Failed to open printer connection.")
            End If

            buffer = System.Text.ASCIIEncoding.GetEncoding(1252).GetBytes(TextBox.Text + Chr(10)) ' Chr(10) is line feed 

            If printer.WritePrinterDirect(buffer, buffer.Length) < 0 Then
                Throw New Exception("Failed to send data to printer.")
            End If

            If printer.ClosePrinter() < 0 Then
                Throw New Exception("Failed to close printer connection.")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1)
        Finally
            printer.Dispose()
        End Try
    End Sub

    Private Sub ButtonPrintTagged_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrintTagged.Click
        Dim printer As New Printer
        Dim buffer As Byte()
        Dim sbt As System.Text.StringBuilder
        Dim i As Integer

        Try
            If printer.OpenPrinter(False, "") < 0 Then
                Throw New Exception("Failed to open printer connection.")
            End If

            ' Remove Chr(13) and Chr(10) because printer shall interpretate them like commands
            sbt = New System.Text.StringBuilder(TextBox.Text.Length)
            For i = 0 To TextBox.Text.Length - 1 Step 1
                If TextBox.Text(i) <> Chr(13) And TextBox.Text(i) <> Chr(10) Then
                    sbt.Append(TextBox.Text(i))
                End If
            Next

            buffer = System.Text.ASCIIEncoding.GetEncoding(1252).GetBytes(sbt.ToString())

            If printer.PrintTaggedText(buffer, buffer.Length) < 0 Then
                Throw New Exception("Failed to print tagged text.")
            End If

            If printer.ClosePrinter() < 0 Then
                Throw New Exception("Failed to close printer connection.")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1)
        Finally
            printer.Dispose()
        End Try
    End Sub

    Private Sub ButtonPrintGraphics_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrintGraphics.Click
        Dim printer As New Printer
        Dim szPage As Printer.Size
        Dim g As Graphics

        Try
            If printer.OpenPrinter(False, "") < 0 Then
                Throw New Exception("Failed to open printer connection.")
            End If

            printer.GetGraphicsPageSize(szPage)
            g = Graphics.FromHdc(printer.GetDC())

            If (printer.StartDoc("") < 0) Then
                Throw New Exception("Failed to start graphics document.")
            End If

            If (printer.StartPage() < 0) Then
                printer.EndDoc(True)
                Throw New Exception("Failed to start graphics page.")
            End If

            g.FillRectangle(New System.Drawing.SolidBrush(Color.White), 0, 0, szPage.cx, szPage.cy)
            g.DrawString(TextBox.Text, New Font(System.Drawing.FontFamily.GenericSerif, NumericUpDownFontSize.Value, FontStyle.Regular), New System.Drawing.SolidBrush(Color.Black), New System.Drawing.RectangleF(0, 0, szPage.cx, szPage.cy))

            If (printer.EndPage() < 0) Then
                printer.EndDoc(True)
                Throw New Exception("Failed to end graphics page.")
            End If

            If (printer.EndDoc(False) < 0) Then
                Throw New Exception("Failed to end  graphics document.")
            End If

            If printer.ClosePrinter() < 0 Then
                Throw New Exception("Failed to close printer connection.")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1)
        Finally
            printer.Dispose()
        End Try
    End Sub

    Private Sub ButtonPrintBarcode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrintBarcode.Click
        Dim printer As New Printer
        Dim barParams As New Printer.PrintBarcodeParams()
        Dim buffer As Byte()

        barParams.eType = comboBoxType.SelectedIndex
        barParams.nHorzSize = Convert.ToInt32(textBoxScale.Text)
        barParams.nHeight = Convert.ToInt32(textBoxHeight.Text)
        barParams.eTextPosition = comboBoxTextPosition.SelectedIndex
        barParams.eTextFont = comboBoxTextSize.SelectedIndex

        ' Defined constants are from 0 to 2 and from 4 to 6 (3 is not present)
        If (comboBoxTextAlign.SelectedIndex < 3) Then
            barParams.eTextAlign = comboBoxTextAlign.SelectedIndex
        Else
            barParams.eTextAlign = comboBoxTextAlign.SelectedIndex + 1
        End If

        buffer = System.Text.Encoding.ASCII.GetBytes(textBoxBarcodeData.Text)

        Try
            If printer.OpenPrinter(False, "") < 0 Then
                Throw New Exception("Failed to open printer connection.")
            End If

            If printer.PrintBarcode(barParams, buffer, buffer.Length) < 0 Then
                Throw New Exception("Failed to print barcode.")
            End If

            If printer.ClosePrinter() < 0 Then
                Throw New Exception("Failed to close printer connection.")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1)
        Finally
            printer.Dispose()
        End Try

    End Sub

    Private Sub ButtonLoadImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonLoadImage.Click
        Dim bitmap As Bitmap

        If OpenFileDialogImage.ShowDialog() = Windows.Forms.DialogResult.OK Then
            bitmap = New Bitmap(OpenFileDialogImage.FileName)
            PictureBoxImage.Image = bitmap
        End If
    End Sub

    Private Sub ButtonPrintImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonPrintImage.Click
        Dim printer As New Printer
        Dim szPage As Printer.Size
        Dim g As Graphics
        Dim offset As Integer
        Dim imageHeight As Integer
        Dim dstRect As Rectangle
        Dim srcRect As Rectangle

        Try
            If PictureBoxImage.Image Is Nothing Then
                Throw New Exception("No image loaded.")
            End If

            If printer.OpenPrinter(False, "") < 0 Then
                Throw New Exception("Failed to open printer connection.")
            End If

            printer.GetGraphicsPageSize(szPage)
            g = Graphics.FromHdc(printer.GetDC())

            If (printer.StartDoc("") < 0) Then
                Throw New Exception("Failed to start graphics document.")
            End If

            offset = 0
            While offset < PictureBoxImage.Image.Height

                If (printer.StartPage() < 0) Then
                    printer.EndDoc(True)
                    Throw New Exception("Failed to start graphics page.")
                End If

                If (PictureBoxImage.Image.Height - offset) < szPage.cy Then
                    imageHeight = PictureBoxImage.Image.Height - offset
                Else
                    imageHeight = szPage.cy
                End If

                dstRect = New Rectangle(0, 0, PictureBoxImage.Image.Width, imageHeight)
                srcRect = New Rectangle(0, offset, PictureBoxImage.Image.Width, imageHeight)

                g.FillRectangle(New System.Drawing.SolidBrush(Color.White), 0, 0, szPage.cx, szPage.cy)
                g.DrawImage(PictureBoxImage.Image, dstRect, srcRect, System.Drawing.GraphicsUnit.Pixel)

                offset = offset + imageHeight

                If (printer.EndPage() < 0) Then
                    printer.EndDoc(True)
                    Throw New Exception("Failed to end graphics page.")
                End If

            End While

            If (printer.EndDoc(False) < 0) Then
                Throw New Exception("Failed to end  graphics document.")
            End If

            If printer.ClosePrinter() < 0 Then
                Throw New Exception("Failed to close printer connection.")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1)
        Finally
            printer.Dispose()
        End Try
    End Sub
End Class
